: ==== start ====
TESTNAME=two-keys-01
source /testing/pluto/bin/eastlocal.sh

ipsec start
/testing/pluto/bin/wait-until-pluto-started

ipsec auto --add westnet-eastnet
ipsec whack --debug none --debug control --debug controlmore --debug crypt
ipsec auto --listpubkeys
