/testing/guestbin/swan-prep
ip addr add 192.1.3.1/24 dev eth0
ip addr add 192.1.3.2/24 dev eth0
ip addr add 192.1.3.3/24 dev eth0
ip addr add 192.1.3.4/24 dev eth0
ip addr add 192.1.3.5/24 dev eth0
ip addr add 192.1.3.6/24 dev eth0
ip addr add 192.1.3.7/24 dev eth0
ip addr add 192.1.3.8/24 dev eth0
ip addr add 192.1.3.9/24 dev eth0
ip addr add 192.1.3.10/24 dev eth0
ip addr add 192.1.3.11/24 dev eth0
ip addr add 192.1.3.12/24 dev eth0
ip addr add 192.1.3.13/24 dev eth0
ip addr add 192.1.3.14/24 dev eth0
ip addr add 192.1.3.15/24 dev eth0
ip addr add 192.1.3.16/24 dev eth0
ip addr add 192.1.3.17/24 dev eth0
ip addr add 192.1.3.18/24 dev eth0
ip addr add 192.1.3.19/24 dev eth0
ip addr add 192.1.3.20/24 dev eth0
ip addr add 192.1.3.21/24 dev eth0
ip addr add 192.1.3.22/24 dev eth0
ip addr add 192.1.3.23/24 dev eth0
ip addr add 192.1.3.24/24 dev eth0
ip addr add 192.1.3.25/24 dev eth0
ip addr add 192.1.3.26/24 dev eth0
ip addr add 192.1.3.27/24 dev eth0
ip addr add 192.1.3.28/24 dev eth0
ip addr add 192.1.3.29/24 dev eth0
ip addr add 192.1.3.30/24 dev eth0
ip addr add 192.1.3.31/24 dev eth0
ip addr add 192.1.3.32/24 dev eth0
ip addr add 192.1.3.33/24 dev eth0
ip addr add 192.1.3.34/24 dev eth0
ip addr add 192.1.3.35/24 dev eth0
ip addr add 192.1.3.36/24 dev eth0
ip addr add 192.1.3.37/24 dev eth0
ip addr add 192.1.3.38/24 dev eth0
ip addr add 192.1.3.39/24 dev eth0
ip addr add 192.1.3.40/24 dev eth0
ip addr add 192.1.3.41/24 dev eth0
ip addr add 192.1.3.42/24 dev eth0
ip addr add 192.1.3.43/24 dev eth0
ip addr add 192.1.3.44/24 dev eth0
ip addr add 192.1.3.45/24 dev eth0
ip addr add 192.1.3.46/24 dev eth0
ip addr add 192.1.3.47/24 dev eth0
ip addr add 192.1.3.48/24 dev eth0
ip addr add 192.1.3.49/24 dev eth0
ip addr add 192.1.3.50/24 dev eth0
ip addr add 192.1.3.51/24 dev eth0
ip addr add 192.1.3.52/24 dev eth0
ip addr add 192.1.3.53/24 dev eth0
ip addr add 192.1.3.54/24 dev eth0
ip addr add 192.1.3.55/24 dev eth0
ip addr add 192.1.3.56/24 dev eth0
ip addr add 192.1.3.57/24 dev eth0
ip addr add 192.1.3.58/24 dev eth0
ip addr add 192.1.3.59/24 dev eth0
ip addr add 192.1.3.60/24 dev eth0
ip addr add 192.1.3.61/24 dev eth0
ip addr add 192.1.3.62/24 dev eth0
ip addr add 192.1.3.63/24 dev eth0
ip addr add 192.1.3.64/24 dev eth0
ip addr add 192.1.3.65/24 dev eth0
ip addr add 192.1.3.66/24 dev eth0
ip addr add 192.1.3.67/24 dev eth0
ip addr add 192.1.3.68/24 dev eth0
ip addr add 192.1.3.69/24 dev eth0
ip addr add 192.1.3.70/24 dev eth0
ip addr add 192.1.3.71/24 dev eth0
ip addr add 192.1.3.72/24 dev eth0
ip addr add 192.1.3.73/24 dev eth0
ip addr add 192.1.3.74/24 dev eth0
ip addr add 192.1.3.75/24 dev eth0
ip addr add 192.1.3.76/24 dev eth0
ip addr add 192.1.3.77/24 dev eth0
ip addr add 192.1.3.78/24 dev eth0
ip addr add 192.1.3.79/24 dev eth0
ip addr add 192.1.3.80/24 dev eth0
ip addr add 192.1.3.81/24 dev eth0
ip addr add 192.1.3.82/24 dev eth0
ip addr add 192.1.3.83/24 dev eth0
ip addr add 192.1.3.84/24 dev eth0
ip addr add 192.1.3.85/24 dev eth0
ip addr add 192.1.3.86/24 dev eth0
ip addr add 192.1.3.87/24 dev eth0
ip addr add 192.1.3.88/24 dev eth0
ip addr add 192.1.3.89/24 dev eth0
ip addr add 192.1.3.90/24 dev eth0
ip addr add 192.1.3.91/24 dev eth0
ip addr add 192.1.3.92/24 dev eth0
ip addr add 192.1.3.93/24 dev eth0
ip addr add 192.1.3.94/24 dev eth0
ip addr add 192.1.3.95/24 dev eth0
ip addr add 192.1.3.96/24 dev eth0
ip addr add 192.1.3.97/24 dev eth0
ip addr add 192.1.3.98/24 dev eth0
ip addr add 192.1.3.99/24 dev eth0
ip addr add 192.1.3.100/24 dev eth0
ip addr add 192.1.3.101/24 dev eth0
ip addr add 192.1.3.102/24 dev eth0
ip addr add 192.1.3.103/24 dev eth0
ip addr add 192.1.3.104/24 dev eth0
ip addr add 192.1.3.105/24 dev eth0
ip addr add 192.1.3.106/24 dev eth0
ip addr add 192.1.3.107/24 dev eth0
ip addr add 192.1.3.108/24 dev eth0
ip addr add 192.1.3.109/24 dev eth0
ip addr add 192.1.3.110/24 dev eth0
ip addr add 192.1.3.111/24 dev eth0
ip addr add 192.1.3.112/24 dev eth0
ip addr add 192.1.3.113/24 dev eth0
ip addr add 192.1.3.114/24 dev eth0
ip addr add 192.1.3.115/24 dev eth0
ip addr add 192.1.3.116/24 dev eth0
ip addr add 192.1.3.117/24 dev eth0
ip addr add 192.1.3.118/24 dev eth0
ip addr add 192.1.3.119/24 dev eth0
ip addr add 192.1.3.120/24 dev eth0
ip addr add 192.1.3.121/24 dev eth0
ip addr add 192.1.3.122/24 dev eth0
ip addr add 192.1.3.123/24 dev eth0
ip addr add 192.1.3.124/24 dev eth0
ip addr add 192.1.3.125/24 dev eth0
ip addr add 192.1.3.126/24 dev eth0
ip addr add 192.1.3.127/24 dev eth0
ip addr add 192.1.3.128/24 dev eth0
ip addr add 192.1.3.129/24 dev eth0
ip addr add 192.1.3.130/24 dev eth0
ip addr add 192.1.3.131/24 dev eth0
ip addr add 192.1.3.132/24 dev eth0
ip addr add 192.1.3.133/24 dev eth0
ip addr add 192.1.3.134/24 dev eth0
ip addr add 192.1.3.135/24 dev eth0
ip addr add 192.1.3.136/24 dev eth0
ip addr add 192.1.3.137/24 dev eth0
ip addr add 192.1.3.138/24 dev eth0
ip addr add 192.1.3.139/24 dev eth0
ip addr add 192.1.3.140/24 dev eth0
ip addr add 192.1.3.141/24 dev eth0
ip addr add 192.1.3.142/24 dev eth0
ip addr add 192.1.3.143/24 dev eth0
ip addr add 192.1.3.144/24 dev eth0
ip addr add 192.1.3.145/24 dev eth0
ip addr add 192.1.3.146/24 dev eth0
ip addr add 192.1.3.147/24 dev eth0
ip addr add 192.1.3.148/24 dev eth0
ip addr add 192.1.3.149/24 dev eth0
ip addr add 192.1.3.150/24 dev eth0
ip addr add 192.1.3.151/24 dev eth0
ip addr add 192.1.3.152/24 dev eth0
ip addr add 192.1.3.153/24 dev eth0
ip addr add 192.1.3.154/24 dev eth0
ip addr add 192.1.3.155/24 dev eth0
ip addr add 192.1.3.156/24 dev eth0
ip addr add 192.1.3.157/24 dev eth0
ip addr add 192.1.3.158/24 dev eth0
ip addr add 192.1.3.159/24 dev eth0
ip addr add 192.1.3.160/24 dev eth0
ip addr add 192.1.3.161/24 dev eth0
ip addr add 192.1.3.162/24 dev eth0
ip addr add 192.1.3.163/24 dev eth0
ip addr add 192.1.3.164/24 dev eth0
ip addr add 192.1.3.165/24 dev eth0
ip addr add 192.1.3.166/24 dev eth0
ip addr add 192.1.3.167/24 dev eth0
ip addr add 192.1.3.168/24 dev eth0
ip addr add 192.1.3.169/24 dev eth0
ip addr add 192.1.3.170/24 dev eth0
ip addr add 192.1.3.171/24 dev eth0
ip addr add 192.1.3.172/24 dev eth0
ip addr add 192.1.3.173/24 dev eth0
ip addr add 192.1.3.174/24 dev eth0
ip addr add 192.1.3.175/24 dev eth0
ip addr add 192.1.3.176/24 dev eth0
ip addr add 192.1.3.177/24 dev eth0
ip addr add 192.1.3.178/24 dev eth0
ip addr add 192.1.3.179/24 dev eth0
ip addr add 192.1.3.180/24 dev eth0
ip addr add 192.1.3.181/24 dev eth0
ip addr add 192.1.3.182/24 dev eth0
ip addr add 192.1.3.183/24 dev eth0
ip addr add 192.1.3.184/24 dev eth0
ip addr add 192.1.3.185/24 dev eth0
ip addr add 192.1.3.186/24 dev eth0
ip addr add 192.1.3.187/24 dev eth0
ip addr add 192.1.3.188/24 dev eth0
ip addr add 192.1.3.189/24 dev eth0
ip addr add 192.1.3.190/24 dev eth0
ip addr add 192.1.3.191/24 dev eth0
ip addr add 192.1.3.192/24 dev eth0
ip addr add 192.1.3.193/24 dev eth0
ip addr add 192.1.3.194/24 dev eth0
ip addr add 192.1.3.195/24 dev eth0
ip addr add 192.1.3.196/24 dev eth0
ip addr add 192.1.3.197/24 dev eth0
ip addr add 192.1.3.198/24 dev eth0
ip addr add 192.1.3.199/24 dev eth0
ip addr add 192.1.3.200/24 dev eth0
ip addr add 192.1.3.201/24 dev eth0
ip addr add 192.1.3.202/24 dev eth0
ip addr add 192.1.3.203/24 dev eth0
ip addr add 192.1.3.204/24 dev eth0
ip addr add 192.1.3.205/24 dev eth0
ip addr add 192.1.3.206/24 dev eth0
ip addr add 192.1.3.207/24 dev eth0
ip addr add 192.1.3.208/24 dev eth0
ip addr add 192.1.3.209/24 dev eth0
ip addr add 192.1.3.210/24 dev eth0
ip addr add 192.1.3.211/24 dev eth0
ip addr add 192.1.3.212/24 dev eth0
ip addr add 192.1.3.213/24 dev eth0
ip addr add 192.1.3.214/24 dev eth0
ip addr add 192.1.3.215/24 dev eth0
ip addr add 192.1.3.216/24 dev eth0
ip addr add 192.1.3.217/24 dev eth0
ip addr add 192.1.3.218/24 dev eth0
ip addr add 192.1.3.219/24 dev eth0
ip addr add 192.1.3.220/24 dev eth0
ip addr add 192.1.3.221/24 dev eth0
ip addr add 192.1.3.222/24 dev eth0
ip addr add 192.1.3.223/24 dev eth0
ip addr add 192.1.3.224/24 dev eth0
ip addr add 192.1.3.225/24 dev eth0
ip addr add 192.1.3.226/24 dev eth0
ip addr add 192.1.3.227/24 dev eth0
ip addr add 192.1.3.228/24 dev eth0
ip addr add 192.1.3.229/24 dev eth0
ip addr add 192.1.3.230/24 dev eth0
ip addr add 192.1.3.231/24 dev eth0
ip addr add 192.1.3.232/24 dev eth0
ip addr add 192.1.3.233/24 dev eth0
ip addr add 192.1.3.234/24 dev eth0
ip addr add 192.1.3.235/24 dev eth0
ip addr add 192.1.3.236/24 dev eth0
ip addr add 192.1.3.237/24 dev eth0
ip addr add 192.1.3.238/24 dev eth0
ip addr add 192.1.3.239/24 dev eth0
ip addr add 192.1.3.240/24 dev eth0
ip addr add 192.1.3.241/24 dev eth0
ip addr add 192.1.3.242/24 dev eth0
ip addr add 192.1.3.243/24 dev eth0
ip addr add 192.1.3.244/24 dev eth0
ip addr add 192.1.3.245/24 dev eth0
ip addr add 192.1.3.246/24 dev eth0
ip addr add 192.1.3.247/24 dev eth0
ip addr add 192.1.3.248/24 dev eth0
ip addr add 192.1.3.249/24 dev eth0
ip addr add 192.1.3.250/24 dev eth0
ip addr add 192.1.3.251/24 dev eth0
ip addr add 192.1.3.252/24 dev eth0
ip addr add 192.1.3.253/24 dev eth0
ipsec start
ipsec whack --impair retransmits
/testing/pluto/bin/wait-until-pluto-started
echo "initdone"
