#!/bin/sh

REF_CONSOLE_FIXUPS="kern-list-fixups.sed nocr.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS cut-postfinal.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS host-prompt-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS east-prompt-splitline.pl"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS cutout.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS wilog.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS klips-debug-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-setup-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS pluto-whack-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS host-ping-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-look-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-look-esp-sanitize.pl"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-ver-remove.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-status.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS netkey-xfrm-sanitizer.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS selinux.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS systemd-fixup.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS pluto-log-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS retransmit-sanitize.sed"
# always included so we can hot-swap libreswan for openswan in any test
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS openswan.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS temp-selinux-ignore.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS misc-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS tcpdump.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ikev2-proposal-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS seccomp.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS all-date-sanitize.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS debug.sed"
