/*
 * Copyright (C) 2003 Michael Richardson <mcr@sandelman.ottawa.on.ca>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#ifndef _IPSEC_IPIP_H_

#ifndef IPPROTO_IPIP
#define IPPROTO_IPIP 4
#endif /* IPPROTO_ESP */

extern struct xform_functions ipip_xform_funcs[];

#define _IPSEC_IPIP_H_

#endif /* _IPSEC_IPIP_H_ */
