ipsec auto --asynchronous --up road-eastnet-nonat-1
ipsec auto --asynchronous --up road-eastnet-nonat-1
ipsec auto --asynchronous --up road-eastnet-nonat-2
ipsec auto --asynchronous --up road-eastnet-nonat-3
ipsec auto --asynchronous --up road-eastnet-nonat-4
ipsec auto --asynchronous --up road-eastnet-nonat-5
ipsec auto --asynchronous --up road-eastnet-nonat-6
ipsec auto --asynchronous --up road-eastnet-nonat-7
ipsec auto --asynchronous --up road-eastnet-nonat-8
ipsec auto --asynchronous --up road-eastnet-nonat-9
ipsec auto --asynchronous --up road-eastnet-nonat-10
ipsec auto --asynchronous --up road-eastnet-nonat-11
ipsec auto --asynchronous --up road-eastnet-nonat-12
ipsec auto --asynchronous --up road-eastnet-nonat-13
ipsec auto --asynchronous --up road-eastnet-nonat-14
ipsec auto --asynchronous --up road-eastnet-nonat-15
ipsec auto --asynchronous --up road-eastnet-nonat-16
ipsec auto --asynchronous --up road-eastnet-nonat-17
ipsec auto --asynchronous --up road-eastnet-nonat-18
ipsec auto --asynchronous --up road-eastnet-nonat-19
ipsec auto --asynchronous --up road-eastnet-nonat-20
ipsec auto --asynchronous --up road-eastnet-nonat-21
ipsec auto --asynchronous --up road-eastnet-nonat-22
ipsec auto --asynchronous --up road-eastnet-nonat-23
ipsec auto --asynchronous --up road-eastnet-nonat-24
ipsec auto --asynchronous --up road-eastnet-nonat-25
ipsec auto --asynchronous --up road-eastnet-nonat-26
ipsec auto --asynchronous --up road-eastnet-nonat-27
ipsec auto --asynchronous --up road-eastnet-nonat-28
ipsec auto --asynchronous --up road-eastnet-nonat-29
ipsec auto --asynchronous --up road-eastnet-nonat-30
ipsec auto --asynchronous --up road-eastnet-nonat-31
ipsec auto --asynchronous --up road-eastnet-nonat-32
ipsec auto --asynchronous --up road-eastnet-nonat-33
ipsec auto --asynchronous --up road-eastnet-nonat-34
ipsec auto --asynchronous --up road-eastnet-nonat-35
ipsec auto --asynchronous --up road-eastnet-nonat-36
ipsec auto --asynchronous --up road-eastnet-nonat-37
ipsec auto --asynchronous --up road-eastnet-nonat-38
ipsec auto --asynchronous --up road-eastnet-nonat-39
ipsec auto --asynchronous --up road-eastnet-nonat-40
ipsec auto --asynchronous --up road-eastnet-nonat-41
ipsec auto --asynchronous --up road-eastnet-nonat-42
ipsec auto --asynchronous --up road-eastnet-nonat-43
ipsec auto --asynchronous --up road-eastnet-nonat-44
ipsec auto --asynchronous --up road-eastnet-nonat-45
ipsec auto --asynchronous --up road-eastnet-nonat-46
ipsec auto --asynchronous --up road-eastnet-nonat-47
ipsec auto --asynchronous --up road-eastnet-nonat-48
ipsec auto --asynchronous --up road-eastnet-nonat-49
ipsec auto --asynchronous --up road-eastnet-nonat-50
ipsec auto --asynchronous --up road-eastnet-nonat-51
ipsec auto --asynchronous --up road-eastnet-nonat-52
ipsec auto --asynchronous --up road-eastnet-nonat-53
ipsec auto --asynchronous --up road-eastnet-nonat-54
ipsec auto --asynchronous --up road-eastnet-nonat-55
ipsec auto --asynchronous --up road-eastnet-nonat-56
ipsec auto --asynchronous --up road-eastnet-nonat-57
ipsec auto --asynchronous --up road-eastnet-nonat-58
ipsec auto --asynchronous --up road-eastnet-nonat-59
ipsec auto --asynchronous --up road-eastnet-nonat-60
ipsec auto --asynchronous --up road-eastnet-nonat-61
ipsec auto --asynchronous --up road-eastnet-nonat-62
ipsec auto --asynchronous --up road-eastnet-nonat-63
ipsec auto --asynchronous --up road-eastnet-nonat-64
ipsec auto --asynchronous --up road-eastnet-nonat-65
ipsec auto --asynchronous --up road-eastnet-nonat-66
ipsec auto --asynchronous --up road-eastnet-nonat-67
ipsec auto --asynchronous --up road-eastnet-nonat-68
ipsec auto --asynchronous --up road-eastnet-nonat-69
ipsec auto --asynchronous --up road-eastnet-nonat-70
ipsec auto --asynchronous --up road-eastnet-nonat-71
ipsec auto --asynchronous --up road-eastnet-nonat-72
ipsec auto --asynchronous --up road-eastnet-nonat-73
ipsec auto --asynchronous --up road-eastnet-nonat-74
ipsec auto --asynchronous --up road-eastnet-nonat-75
ipsec auto --asynchronous --up road-eastnet-nonat-76
ipsec auto --asynchronous --up road-eastnet-nonat-77
ipsec auto --asynchronous --up road-eastnet-nonat-78
ipsec auto --asynchronous --up road-eastnet-nonat-79
ipsec auto --asynchronous --up road-eastnet-nonat-80
ipsec auto --asynchronous --up road-eastnet-nonat-81
ipsec auto --asynchronous --up road-eastnet-nonat-82
ipsec auto --asynchronous --up road-eastnet-nonat-83
ipsec auto --asynchronous --up road-eastnet-nonat-84
ipsec auto --asynchronous --up road-eastnet-nonat-85
ipsec auto --asynchronous --up road-eastnet-nonat-86
ipsec auto --asynchronous --up road-eastnet-nonat-87
ipsec auto --asynchronous --up road-eastnet-nonat-88
ipsec auto --asynchronous --up road-eastnet-nonat-89
ipsec auto --asynchronous --up road-eastnet-nonat-90
ipsec auto --asynchronous --up road-eastnet-nonat-91
ipsec auto --asynchronous --up road-eastnet-nonat-92
ipsec auto --asynchronous --up road-eastnet-nonat-93
ipsec auto --asynchronous --up road-eastnet-nonat-94
ipsec auto --asynchronous --up road-eastnet-nonat-95
ipsec auto --asynchronous --up road-eastnet-nonat-96
ipsec auto --asynchronous --up road-eastnet-nonat-97
ipsec auto --asynchronous --up road-eastnet-nonat-98
ipsec auto --asynchronous --up road-eastnet-nonat-99
ipsec auto --asynchronous --up road-eastnet-nonat-100
ipsec auto --asynchronous --up road-eastnet-nonat-101
ipsec auto --asynchronous --up road-eastnet-nonat-102
ipsec auto --asynchronous --up road-eastnet-nonat-103
ipsec auto --asynchronous --up road-eastnet-nonat-104
ipsec auto --asynchronous --up road-eastnet-nonat-105
ipsec auto --asynchronous --up road-eastnet-nonat-106
ipsec auto --asynchronous --up road-eastnet-nonat-107
ipsec auto --asynchronous --up road-eastnet-nonat-108
ipsec auto --asynchronous --up road-eastnet-nonat-109
ipsec auto --asynchronous --up road-eastnet-nonat-110
ipsec auto --asynchronous --up road-eastnet-nonat-111
ipsec auto --asynchronous --up road-eastnet-nonat-112
ipsec auto --asynchronous --up road-eastnet-nonat-113
ipsec auto --asynchronous --up road-eastnet-nonat-114
ipsec auto --asynchronous --up road-eastnet-nonat-115
ipsec auto --asynchronous --up road-eastnet-nonat-116
ipsec auto --asynchronous --up road-eastnet-nonat-117
ipsec auto --asynchronous --up road-eastnet-nonat-118
ipsec auto --asynchronous --up road-eastnet-nonat-119
ipsec auto --asynchronous --up road-eastnet-nonat-120
ipsec auto --asynchronous --up road-eastnet-nonat-121
ipsec auto --asynchronous --up road-eastnet-nonat-122
ipsec auto --asynchronous --up road-eastnet-nonat-123
ipsec auto --asynchronous --up road-eastnet-nonat-124
ipsec auto --asynchronous --up road-eastnet-nonat-125
ipsec auto --asynchronous --up road-eastnet-nonat-126
ipsec auto --asynchronous --up road-eastnet-nonat-127
ipsec auto --asynchronous --up road-eastnet-nonat-128
ipsec auto --asynchronous --up road-eastnet-nonat-129
ipsec auto --asynchronous --up road-eastnet-nonat-130
ipsec auto --asynchronous --up road-eastnet-nonat-131
ipsec auto --asynchronous --up road-eastnet-nonat-132
ipsec auto --asynchronous --up road-eastnet-nonat-133
ipsec auto --asynchronous --up road-eastnet-nonat-134
ipsec auto --asynchronous --up road-eastnet-nonat-135
ipsec auto --asynchronous --up road-eastnet-nonat-136
ipsec auto --asynchronous --up road-eastnet-nonat-137
ipsec auto --asynchronous --up road-eastnet-nonat-138
ipsec auto --asynchronous --up road-eastnet-nonat-139
ipsec auto --asynchronous --up road-eastnet-nonat-140
ipsec auto --asynchronous --up road-eastnet-nonat-141
ipsec auto --asynchronous --up road-eastnet-nonat-142
ipsec auto --asynchronous --up road-eastnet-nonat-143
ipsec auto --asynchronous --up road-eastnet-nonat-144
ipsec auto --asynchronous --up road-eastnet-nonat-145
ipsec auto --asynchronous --up road-eastnet-nonat-146
ipsec auto --asynchronous --up road-eastnet-nonat-147
ipsec auto --asynchronous --up road-eastnet-nonat-148
ipsec auto --asynchronous --up road-eastnet-nonat-149
ipsec auto --asynchronous --up road-eastnet-nonat-150
ipsec auto --asynchronous --up road-eastnet-nonat-151
ipsec auto --asynchronous --up road-eastnet-nonat-152
ipsec auto --asynchronous --up road-eastnet-nonat-153
ipsec auto --asynchronous --up road-eastnet-nonat-154
ipsec auto --asynchronous --up road-eastnet-nonat-155
ipsec auto --asynchronous --up road-eastnet-nonat-156
ipsec auto --asynchronous --up road-eastnet-nonat-157
ipsec auto --asynchronous --up road-eastnet-nonat-158
ipsec auto --asynchronous --up road-eastnet-nonat-159
ipsec auto --asynchronous --up road-eastnet-nonat-160
ipsec auto --asynchronous --up road-eastnet-nonat-161
ipsec auto --asynchronous --up road-eastnet-nonat-162
ipsec auto --asynchronous --up road-eastnet-nonat-163
ipsec auto --asynchronous --up road-eastnet-nonat-164
ipsec auto --asynchronous --up road-eastnet-nonat-165
ipsec auto --asynchronous --up road-eastnet-nonat-166
ipsec auto --asynchronous --up road-eastnet-nonat-167
ipsec auto --asynchronous --up road-eastnet-nonat-168
ipsec auto --asynchronous --up road-eastnet-nonat-169
ipsec auto --asynchronous --up road-eastnet-nonat-170
ipsec auto --asynchronous --up road-eastnet-nonat-171
ipsec auto --asynchronous --up road-eastnet-nonat-172
ipsec auto --asynchronous --up road-eastnet-nonat-173
ipsec auto --asynchronous --up road-eastnet-nonat-174
ipsec auto --asynchronous --up road-eastnet-nonat-175
ipsec auto --asynchronous --up road-eastnet-nonat-176
ipsec auto --asynchronous --up road-eastnet-nonat-177
ipsec auto --asynchronous --up road-eastnet-nonat-178
ipsec auto --asynchronous --up road-eastnet-nonat-179
ipsec auto --asynchronous --up road-eastnet-nonat-180
ipsec auto --asynchronous --up road-eastnet-nonat-181
ipsec auto --asynchronous --up road-eastnet-nonat-182
ipsec auto --asynchronous --up road-eastnet-nonat-183
ipsec auto --asynchronous --up road-eastnet-nonat-184
ipsec auto --asynchronous --up road-eastnet-nonat-185
ipsec auto --asynchronous --up road-eastnet-nonat-186
ipsec auto --asynchronous --up road-eastnet-nonat-187
ipsec auto --asynchronous --up road-eastnet-nonat-188
ipsec auto --asynchronous --up road-eastnet-nonat-189
ipsec auto --asynchronous --up road-eastnet-nonat-190
ipsec auto --asynchronous --up road-eastnet-nonat-191
ipsec auto --asynchronous --up road-eastnet-nonat-192
ipsec auto --asynchronous --up road-eastnet-nonat-193
ipsec auto --asynchronous --up road-eastnet-nonat-194
ipsec auto --asynchronous --up road-eastnet-nonat-195
ipsec auto --asynchronous --up road-eastnet-nonat-196
ipsec auto --asynchronous --up road-eastnet-nonat-197
ipsec auto --asynchronous --up road-eastnet-nonat-198
ipsec auto --asynchronous --up road-eastnet-nonat-199
ipsec auto --asynchronous --up road-eastnet-nonat-200
ipsec auto --asynchronous --up road-eastnet-nonat-201
ipsec auto --asynchronous --up road-eastnet-nonat-202
ipsec auto --asynchronous --up road-eastnet-nonat-203
ipsec auto --asynchronous --up road-eastnet-nonat-204
ipsec auto --asynchronous --up road-eastnet-nonat-205
ipsec auto --asynchronous --up road-eastnet-nonat-206
ipsec auto --asynchronous --up road-eastnet-nonat-207
ipsec auto --asynchronous --up road-eastnet-nonat-208
ipsec auto --asynchronous --up road-eastnet-nonat-209
ipsec auto --asynchronous --up road-eastnet-nonat-210
ipsec auto --asynchronous --up road-eastnet-nonat-211
ipsec auto --asynchronous --up road-eastnet-nonat-212
ipsec auto --asynchronous --up road-eastnet-nonat-213
ipsec auto --asynchronous --up road-eastnet-nonat-214
ipsec auto --asynchronous --up road-eastnet-nonat-215
ipsec auto --asynchronous --up road-eastnet-nonat-216
ipsec auto --asynchronous --up road-eastnet-nonat-217
ipsec auto --asynchronous --up road-eastnet-nonat-218
ipsec auto --asynchronous --up road-eastnet-nonat-219
ipsec auto --asynchronous --up road-eastnet-nonat-220
ipsec auto --asynchronous --up road-eastnet-nonat-221
ipsec auto --asynchronous --up road-eastnet-nonat-222
ipsec auto --asynchronous --up road-eastnet-nonat-223
ipsec auto --asynchronous --up road-eastnet-nonat-224
ipsec auto --asynchronous --up road-eastnet-nonat-225
ipsec auto --asynchronous --up road-eastnet-nonat-226
ipsec auto --asynchronous --up road-eastnet-nonat-227
ipsec auto --asynchronous --up road-eastnet-nonat-228
ipsec auto --asynchronous --up road-eastnet-nonat-229
ipsec auto --asynchronous --up road-eastnet-nonat-230
ipsec auto --asynchronous --up road-eastnet-nonat-231
ipsec auto --asynchronous --up road-eastnet-nonat-232
ipsec auto --asynchronous --up road-eastnet-nonat-233
ipsec auto --asynchronous --up road-eastnet-nonat-234
ipsec auto --asynchronous --up road-eastnet-nonat-235
ipsec auto --asynchronous --up road-eastnet-nonat-236
ipsec auto --asynchronous --up road-eastnet-nonat-237
ipsec auto --asynchronous --up road-eastnet-nonat-238
ipsec auto --asynchronous --up road-eastnet-nonat-239
ipsec auto --asynchronous --up road-eastnet-nonat-240
ipsec auto --asynchronous --up road-eastnet-nonat-241
ipsec auto --asynchronous --up road-eastnet-nonat-242
ipsec auto --asynchronous --up road-eastnet-nonat-243
ipsec auto --asynchronous --up road-eastnet-nonat-244
ipsec auto --asynchronous --up road-eastnet-nonat-245
ipsec auto --asynchronous --up road-eastnet-nonat-246
ipsec auto --asynchronous --up road-eastnet-nonat-247
ipsec auto --asynchronous --up road-eastnet-nonat-248
ipsec auto --asynchronous --up road-eastnet-nonat-249
ipsec auto --asynchronous --up road-eastnet-nonat-250
ipsec auto --asynchronous --up road-eastnet-nonat-251
ipsec auto --asynchronous --up road-eastnet-nonat-252
ipsec auto --asynchronous --up road-eastnet-nonat-253
ping -q -c 2 -I 192.1.3.1 192.1.2.23
ping -q -c 2 -I 192.1.3.2 192.1.2.23
ping -q -c 2 -I 192.1.3.3 192.1.2.23
ping -q -c 2 -I 192.1.3.4 192.1.2.23
ping -q -c 2 -I 192.1.3.5 192.1.2.23
ping -q -c 2 -I 192.1.3.6 192.1.2.23
ping -q -c 2 -I 192.1.3.7 192.1.2.23
ping -q -c 2 -I 192.1.3.8 192.1.2.23
ping -q -c 2 -I 192.1.3.9 192.1.2.23
ping -q -c 2 -I 192.1.3.10 192.1.2.23
ping -q -c 2 -I 192.1.3.11 192.1.2.23
ping -q -c 2 -I 192.1.3.12 192.1.2.23
ping -q -c 2 -I 192.1.3.13 192.1.2.23
ping -q -c 2 -I 192.1.3.14 192.1.2.23
ping -q -c 2 -I 192.1.3.15 192.1.2.23
ping -q -c 2 -I 192.1.3.16 192.1.2.23
ping -q -c 2 -I 192.1.3.17 192.1.2.23
ping -q -c 2 -I 192.1.3.18 192.1.2.23
ping -q -c 2 -I 192.1.3.19 192.1.2.23
ping -q -c 2 -I 192.1.3.20 192.1.2.23
ping -q -c 2 -I 192.1.3.21 192.1.2.23
ping -q -c 2 -I 192.1.3.22 192.1.2.23
ping -q -c 2 -I 192.1.3.23 192.1.2.23
ping -q -c 2 -I 192.1.3.24 192.1.2.23
ping -q -c 2 -I 192.1.3.25 192.1.2.23
ping -q -c 2 -I 192.1.3.26 192.1.2.23
ping -q -c 2 -I 192.1.3.27 192.1.2.23
ping -q -c 2 -I 192.1.3.28 192.1.2.23
ping -q -c 2 -I 192.1.3.29 192.1.2.23
ping -q -c 2 -I 192.1.3.30 192.1.2.23
ping -q -c 2 -I 192.1.3.31 192.1.2.23
ping -q -c 2 -I 192.1.3.32 192.1.2.23
ping -q -c 2 -I 192.1.3.33 192.1.2.23
ping -q -c 2 -I 192.1.3.34 192.1.2.23
ping -q -c 2 -I 192.1.3.35 192.1.2.23
ping -q -c 2 -I 192.1.3.36 192.1.2.23
ping -q -c 2 -I 192.1.3.37 192.1.2.23
ping -q -c 2 -I 192.1.3.38 192.1.2.23
ping -q -c 2 -I 192.1.3.39 192.1.2.23
ping -q -c 2 -I 192.1.3.40 192.1.2.23
ping -q -c 2 -I 192.1.3.41 192.1.2.23
ping -q -c 2 -I 192.1.3.42 192.1.2.23
ping -q -c 2 -I 192.1.3.43 192.1.2.23
ping -q -c 2 -I 192.1.3.44 192.1.2.23
ping -q -c 2 -I 192.1.3.45 192.1.2.23
ping -q -c 2 -I 192.1.3.46 192.1.2.23
ping -q -c 2 -I 192.1.3.47 192.1.2.23
ping -q -c 2 -I 192.1.3.48 192.1.2.23
ping -q -c 2 -I 192.1.3.49 192.1.2.23
ping -q -c 2 -I 192.1.3.50 192.1.2.23
ping -q -c 2 -I 192.1.3.51 192.1.2.23
ping -q -c 2 -I 192.1.3.52 192.1.2.23
ping -q -c 2 -I 192.1.3.53 192.1.2.23
ping -q -c 2 -I 192.1.3.54 192.1.2.23
ping -q -c 2 -I 192.1.3.55 192.1.2.23
ping -q -c 2 -I 192.1.3.56 192.1.2.23
ping -q -c 2 -I 192.1.3.57 192.1.2.23
ping -q -c 2 -I 192.1.3.58 192.1.2.23
ping -q -c 2 -I 192.1.3.59 192.1.2.23
ping -q -c 2 -I 192.1.3.60 192.1.2.23
ping -q -c 2 -I 192.1.3.61 192.1.2.23
ping -q -c 2 -I 192.1.3.62 192.1.2.23
ping -q -c 2 -I 192.1.3.63 192.1.2.23
ping -q -c 2 -I 192.1.3.64 192.1.2.23
ping -q -c 2 -I 192.1.3.65 192.1.2.23
ping -q -c 2 -I 192.1.3.66 192.1.2.23
ping -q -c 2 -I 192.1.3.67 192.1.2.23
ping -q -c 2 -I 192.1.3.68 192.1.2.23
ping -q -c 2 -I 192.1.3.69 192.1.2.23
ping -q -c 2 -I 192.1.3.70 192.1.2.23
ping -q -c 2 -I 192.1.3.71 192.1.2.23
ping -q -c 2 -I 192.1.3.72 192.1.2.23
ping -q -c 2 -I 192.1.3.73 192.1.2.23
ping -q -c 2 -I 192.1.3.74 192.1.2.23
ping -q -c 2 -I 192.1.3.75 192.1.2.23
ping -q -c 2 -I 192.1.3.76 192.1.2.23
ping -q -c 2 -I 192.1.3.77 192.1.2.23
ping -q -c 2 -I 192.1.3.78 192.1.2.23
ping -q -c 2 -I 192.1.3.79 192.1.2.23
ping -q -c 2 -I 192.1.3.80 192.1.2.23
ping -q -c 2 -I 192.1.3.81 192.1.2.23
ping -q -c 2 -I 192.1.3.82 192.1.2.23
ping -q -c 2 -I 192.1.3.83 192.1.2.23
ping -q -c 2 -I 192.1.3.84 192.1.2.23
ping -q -c 2 -I 192.1.3.85 192.1.2.23
ping -q -c 2 -I 192.1.3.86 192.1.2.23
ping -q -c 2 -I 192.1.3.87 192.1.2.23
ping -q -c 2 -I 192.1.3.88 192.1.2.23
ping -q -c 2 -I 192.1.3.89 192.1.2.23
ping -q -c 2 -I 192.1.3.90 192.1.2.23
ping -q -c 2 -I 192.1.3.91 192.1.2.23
ping -q -c 2 -I 192.1.3.92 192.1.2.23
ping -q -c 2 -I 192.1.3.93 192.1.2.23
ping -q -c 2 -I 192.1.3.94 192.1.2.23
ping -q -c 2 -I 192.1.3.95 192.1.2.23
ping -q -c 2 -I 192.1.3.96 192.1.2.23
ping -q -c 2 -I 192.1.3.97 192.1.2.23
ping -q -c 2 -I 192.1.3.98 192.1.2.23
ping -q -c 2 -I 192.1.3.99 192.1.2.23
ping -q -c 2 -I 192.1.3.100 192.1.2.23
ping -q -c 2 -I 192.1.3.101 192.1.2.23
ping -q -c 2 -I 192.1.3.102 192.1.2.23
ping -q -c 2 -I 192.1.3.103 192.1.2.23
ping -q -c 2 -I 192.1.3.104 192.1.2.23
ping -q -c 2 -I 192.1.3.105 192.1.2.23
ping -q -c 2 -I 192.1.3.106 192.1.2.23
ping -q -c 2 -I 192.1.3.107 192.1.2.23
ping -q -c 2 -I 192.1.3.108 192.1.2.23
ping -q -c 2 -I 192.1.3.109 192.1.2.23
ping -q -c 2 -I 192.1.3.110 192.1.2.23
ping -q -c 2 -I 192.1.3.111 192.1.2.23
ping -q -c 2 -I 192.1.3.112 192.1.2.23
ping -q -c 2 -I 192.1.3.113 192.1.2.23
ping -q -c 2 -I 192.1.3.114 192.1.2.23
ping -q -c 2 -I 192.1.3.115 192.1.2.23
ping -q -c 2 -I 192.1.3.116 192.1.2.23
ping -q -c 2 -I 192.1.3.117 192.1.2.23
ping -q -c 2 -I 192.1.3.118 192.1.2.23
ping -q -c 2 -I 192.1.3.119 192.1.2.23
ping -q -c 2 -I 192.1.3.120 192.1.2.23
ping -q -c 2 -I 192.1.3.121 192.1.2.23
ping -q -c 2 -I 192.1.3.122 192.1.2.23
ping -q -c 2 -I 192.1.3.123 192.1.2.23
ping -q -c 2 -I 192.1.3.124 192.1.2.23
ping -q -c 2 -I 192.1.3.125 192.1.2.23
ping -q -c 2 -I 192.1.3.126 192.1.2.23
ping -q -c 2 -I 192.1.3.127 192.1.2.23
ping -q -c 2 -I 192.1.3.128 192.1.2.23
ping -q -c 2 -I 192.1.3.129 192.1.2.23
ping -q -c 2 -I 192.1.3.130 192.1.2.23
ping -q -c 2 -I 192.1.3.131 192.1.2.23
ping -q -c 2 -I 192.1.3.132 192.1.2.23
ping -q -c 2 -I 192.1.3.133 192.1.2.23
ping -q -c 2 -I 192.1.3.134 192.1.2.23
ping -q -c 2 -I 192.1.3.135 192.1.2.23
ping -q -c 2 -I 192.1.3.136 192.1.2.23
ping -q -c 2 -I 192.1.3.137 192.1.2.23
ping -q -c 2 -I 192.1.3.138 192.1.2.23
ping -q -c 2 -I 192.1.3.139 192.1.2.23
ping -q -c 2 -I 192.1.3.140 192.1.2.23
ping -q -c 2 -I 192.1.3.141 192.1.2.23
ping -q -c 2 -I 192.1.3.142 192.1.2.23
ping -q -c 2 -I 192.1.3.143 192.1.2.23
ping -q -c 2 -I 192.1.3.144 192.1.2.23
ping -q -c 2 -I 192.1.3.145 192.1.2.23
ping -q -c 2 -I 192.1.3.146 192.1.2.23
ping -q -c 2 -I 192.1.3.147 192.1.2.23
ping -q -c 2 -I 192.1.3.148 192.1.2.23
ping -q -c 2 -I 192.1.3.149 192.1.2.23
ping -q -c 2 -I 192.1.3.150 192.1.2.23
ping -q -c 2 -I 192.1.3.151 192.1.2.23
ping -q -c 2 -I 192.1.3.152 192.1.2.23
ping -q -c 2 -I 192.1.3.153 192.1.2.23
ping -q -c 2 -I 192.1.3.154 192.1.2.23
ping -q -c 2 -I 192.1.3.155 192.1.2.23
ping -q -c 2 -I 192.1.3.156 192.1.2.23
ping -q -c 2 -I 192.1.3.157 192.1.2.23
ping -q -c 2 -I 192.1.3.158 192.1.2.23
ping -q -c 2 -I 192.1.3.159 192.1.2.23
ping -q -c 2 -I 192.1.3.160 192.1.2.23
ping -q -c 2 -I 192.1.3.161 192.1.2.23
ping -q -c 2 -I 192.1.3.162 192.1.2.23
ping -q -c 2 -I 192.1.3.163 192.1.2.23
ping -q -c 2 -I 192.1.3.164 192.1.2.23
ping -q -c 2 -I 192.1.3.165 192.1.2.23
ping -q -c 2 -I 192.1.3.166 192.1.2.23
ping -q -c 2 -I 192.1.3.167 192.1.2.23
ping -q -c 2 -I 192.1.3.168 192.1.2.23
ping -q -c 2 -I 192.1.3.169 192.1.2.23
ping -q -c 2 -I 192.1.3.170 192.1.2.23
ping -q -c 2 -I 192.1.3.171 192.1.2.23
ping -q -c 2 -I 192.1.3.172 192.1.2.23
ping -q -c 2 -I 192.1.3.173 192.1.2.23
ping -q -c 2 -I 192.1.3.174 192.1.2.23
ping -q -c 2 -I 192.1.3.175 192.1.2.23
ping -q -c 2 -I 192.1.3.176 192.1.2.23
ping -q -c 2 -I 192.1.3.177 192.1.2.23
ping -q -c 2 -I 192.1.3.178 192.1.2.23
ping -q -c 2 -I 192.1.3.179 192.1.2.23
ping -q -c 2 -I 192.1.3.180 192.1.2.23
ping -q -c 2 -I 192.1.3.181 192.1.2.23
ping -q -c 2 -I 192.1.3.182 192.1.2.23
ping -q -c 2 -I 192.1.3.183 192.1.2.23
ping -q -c 2 -I 192.1.3.184 192.1.2.23
ping -q -c 2 -I 192.1.3.185 192.1.2.23
ping -q -c 2 -I 192.1.3.186 192.1.2.23
ping -q -c 2 -I 192.1.3.187 192.1.2.23
ping -q -c 2 -I 192.1.3.188 192.1.2.23
ping -q -c 2 -I 192.1.3.189 192.1.2.23
ping -q -c 2 -I 192.1.3.190 192.1.2.23
ping -q -c 2 -I 192.1.3.191 192.1.2.23
ping -q -c 2 -I 192.1.3.192 192.1.2.23
ping -q -c 2 -I 192.1.3.193 192.1.2.23
ping -q -c 2 -I 192.1.3.194 192.1.2.23
ping -q -c 2 -I 192.1.3.195 192.1.2.23
ping -q -c 2 -I 192.1.3.196 192.1.2.23
ping -q -c 2 -I 192.1.3.197 192.1.2.23
ping -q -c 2 -I 192.1.3.198 192.1.2.23
ping -q -c 2 -I 192.1.3.199 192.1.2.23
ping -q -c 2 -I 192.1.3.200 192.1.2.23
ping -q -c 2 -I 192.1.3.201 192.1.2.23
ping -q -c 2 -I 192.1.3.202 192.1.2.23
ping -q -c 2 -I 192.1.3.203 192.1.2.23
ping -q -c 2 -I 192.1.3.204 192.1.2.23
ping -q -c 2 -I 192.1.3.205 192.1.2.23
ping -q -c 2 -I 192.1.3.206 192.1.2.23
ping -q -c 2 -I 192.1.3.207 192.1.2.23
ping -q -c 2 -I 192.1.3.208 192.1.2.23
ping -q -c 2 -I 192.1.3.209 192.1.2.23
ping -q -c 2 -I 192.1.3.210 192.1.2.23
ping -q -c 2 -I 192.1.3.211 192.1.2.23
ping -q -c 2 -I 192.1.3.212 192.1.2.23
ping -q -c 2 -I 192.1.3.213 192.1.2.23
ping -q -c 2 -I 192.1.3.214 192.1.2.23
ping -q -c 2 -I 192.1.3.215 192.1.2.23
ping -q -c 2 -I 192.1.3.216 192.1.2.23
ping -q -c 2 -I 192.1.3.217 192.1.2.23
ping -q -c 2 -I 192.1.3.218 192.1.2.23
ping -q -c 2 -I 192.1.3.219 192.1.2.23
ping -q -c 2 -I 192.1.3.220 192.1.2.23
ping -q -c 2 -I 192.1.3.221 192.1.2.23
ping -q -c 2 -I 192.1.3.222 192.1.2.23
ping -q -c 2 -I 192.1.3.223 192.1.2.23
ping -q -c 2 -I 192.1.3.224 192.1.2.23
ping -q -c 2 -I 192.1.3.225 192.1.2.23
ping -q -c 2 -I 192.1.3.226 192.1.2.23
ping -q -c 2 -I 192.1.3.227 192.1.2.23
ping -q -c 2 -I 192.1.3.228 192.1.2.23
ping -q -c 2 -I 192.1.3.229 192.1.2.23
ping -q -c 2 -I 192.1.3.230 192.1.2.23
ping -q -c 2 -I 192.1.3.231 192.1.2.23
ping -q -c 2 -I 192.1.3.232 192.1.2.23
ping -q -c 2 -I 192.1.3.233 192.1.2.23
ping -q -c 2 -I 192.1.3.234 192.1.2.23
ping -q -c 2 -I 192.1.3.235 192.1.2.23
ping -q -c 2 -I 192.1.3.236 192.1.2.23
ping -q -c 2 -I 192.1.3.237 192.1.2.23
ping -q -c 2 -I 192.1.3.238 192.1.2.23
ping -q -c 2 -I 192.1.3.239 192.1.2.23
ping -q -c 2 -I 192.1.3.240 192.1.2.23
ping -q -c 2 -I 192.1.3.241 192.1.2.23
ping -q -c 2 -I 192.1.3.242 192.1.2.23
ping -q -c 2 -I 192.1.3.243 192.1.2.23
ping -q -c 2 -I 192.1.3.244 192.1.2.23
ping -q -c 2 -I 192.1.3.245 192.1.2.23
ping -q -c 2 -I 192.1.3.246 192.1.2.23
ping -q -c 2 -I 192.1.3.247 192.1.2.23
ping -q -c 2 -I 192.1.3.248 192.1.2.23
ping -q -c 2 -I 192.1.3.249 192.1.2.23
ping -q -c 2 -I 192.1.3.250 192.1.2.23
ping -q -c 2 -I 192.1.3.251 192.1.2.23
ping -q -c 2 -I 192.1.3.252 192.1.2.23
ping -q -c 2 -I 192.1.3.253 192.1.2.23
ipsec whack --trafficstatus
echo done
